#ifndef _SoftTimer_H_
#define _SoftTimer_H_

/*  ________________________________________________________________________  */
/* |----------------------  P R O J E C T   N O T E S  ---------------------| */
/* |________________________________________________________________________| */
/* |                                                                        | */
/* |                           frydaysoft.de                                | */
/* |                                                                        | */
/* |________________________________________________________________________| */
/* | author     :   Sven Freitag                                            | */
/* |________________________________________________________________________| */
/* |****************************  S O U R C E  *****************************| */
/* |________________________________________________________________________| */
/* | project    :   LightCtrl                                               | */
/* |________________________________________________________________________| */
/* | file name  :   SoftTimer.h      date: 12.11.2009      version :  1.0   | */
/* |________________________________________________________________________| */
/* | task       :   Software Timer module                                   | */
/* |________________________________________________________________________| */
/* | target     :   ATMEL ATmega48/88, 3.6864 MHz                           | */
/* |________________________________________________________________________| */
/* | compiler   :   WIN AVR C Compiler                                      | */
/* |________________________________________________________________________| */
/* | history    :   01.09.2009  project start                               | */
/* |________________________________________________________________________| */
/* | tested by  :                                                           | */
/* |________________________________________________________________________| */


// *****************************************************************************
// ***************************  Include Directives  ****************************
// *****************************************************************************


// *****************************************************************************
// **********************  Global Funtion Prototyps    *************************
// *****************************************************************************




// *****************************************************************************
// ***                                SoftTimerInit()                        ***
// *****************************************************************************
// ***  Initializes Timer2 for 1ms and enables compare match A interrupt.    ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***                                    void                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***                                    void                               ***
// *****************************************************************************

void SoftTimerInit(void);


// *****************************************************************************
// ***                               SoftTimerDeInit()                       ***
// *****************************************************************************
// ***  DeInitializes Timer2.                                                ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***                                    void                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***                                    void                               ***
// *****************************************************************************

void SoftTimerDeInit(void);



// *****************************************************************************
// ***                          ucSoftTimerGetTime1ms()                      ***
// *****************************************************************************
// ***  Function to get the current timecounter value (resolution = 1 ms).   ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***                                    void                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***                        current timecounter (resolution = 1 ms)        ***
// *****************************************************************************

UCHAR ucSoftTimerGetTime1ms(void);


// *****************************************************************************
// ***                        ucSoftTimerGetGetTimeDiff1ms()                 ***
// *****************************************************************************
// ***  Function calculates the difference time (in ms) between the          ***
// ***  'start time' and the current time.                                   ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***    UCHAR uc_start_time_mark  :  start time                            ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***    UCHAR                     :   difference time in 1ms.              ***
// *****************************************************************************

UCHAR ucSoftTimerGetGetTimeDiff1ms(UCHAR uc_start_time_mark);




#endif  // _SoftTimer_H_

// ******************************** End of File ********************************
