#ifndef _Uart_H_
#define _Uart_H_

/*  ________________________________________________________________________  */
/* |----------------------  P R O J E C T   N O T E S  ---------------------| */
/* |________________________________________________________________________| */
/* |                                                                        | */
/* |                           frydaysoft.de                                | */
/* |                                                                        | */
/* |________________________________________________________________________| */
/* | author     :   Sven Freitag                                            | */
/* |________________________________________________________________________| */
/* |****************************  S O U R C E  *****************************| */
/* |________________________________________________________________________| */
/* | project    :   LightCtrl                                               | */
/* |________________________________________________________________________| */
/* | file name  :   Uart.h           date: 12.11.2009      version :  1.0   | */
/* |________________________________________________________________________| */
/* | task       :   Uart module                                             | */
/* |________________________________________________________________________| */
/* | target     :   ATMEL ATmega48/88, 3.6864 MHz                           | */
/* |________________________________________________________________________| */
/* | compiler   :   WIN AVR C Compiler                                      | */
/* |________________________________________________________________________| */
/* | history    :   01.09.2009  project start                               | */
/* |________________________________________________________________________| */
/* | tested by  :                                                           | */
/* |________________________________________________________________________| */

// ******************************************************************************
// ******************  Module Global Definitions and Macros  ********************
// ******************************************************************************

#define RX_MAX_LEN 18     // we have limited RAM so reduce communication protocol
#define RX_CMD_HEADER 2
#define RX_MAX_DATALEN (RX_MAX_LEN-RX_CMD_HEADER)


// ***** some baudrates for 8MHz
//#define BAUDRATE_19200       25
//#define BAUDRATE_9600        51
//#define BAUDRATE_4800        103

// ***** some baudrates for 3,6864MHz
#define BAUDRATE_57600        3
#define BAUDRATE_19200       11
#define BAUDRATE_9600        23
#define BAUDRATE_4800        47


//******************************************************************************
// simple macro to append end framing behinde user data
#define ComAddTailToken(buf, idx)  buf[idx++] = END_TOK


/******************************************************************************/
/***                     UartInit()                                         ***/
/******************************************************************************/
/***  Initializes Hardware Uart                                             ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/***    ui_baudrate :  baudrate                                             ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/******************************************************************************/

void UartInit(UINT ui_baudrate);


/******************************************************************************/
/***                   bUartIsTxIdle()                                      ***/
/******************************************************************************/
/***  Test if uart module ready for transmit new data                       ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    BOOL                 TRUE: ok          FALSE: error                 ***/
/******************************************************************************/

BOOL bUartIsTxIdle(void);

  

/******************************************************************************/
/***                      bUartSendBlock()                                  ***/
/******************************************************************************/
/***  Send block over Uart with special framing                             ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/***    puc_buf : tx buffer                                                 ***/
/***    uc_len :  lenght of tx buffer (0..63)                               ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    BOOL                 TRUE: ok          FALSE: error                 ***/
/******************************************************************************/

BOOL bUartSendBlock( UCHAR *puc_buf, UCHAR uc_len);



/******************************************************************************/
/***                        bUartExistRxData()                              ***/
/******************************************************************************/
/***  Test if complete block received                                       ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    BOOL                 TRUE: ok          FALSE: no                    ***/
/******************************************************************************/

BOOL bUartExistRxData(void);



/******************************************************************************/
/***                        ucUartCopyRxData()                              ***/
/******************************************************************************/
/***  Copy received data into given buffer and restart receiving            ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/***    puc_buf : destination buffer                                        ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    copied length                                                       ***/
/******************************************************************************/

UCHAR ucUartCopyRxData( UCHAR *puc_buf);



#endif  // _Uart_H_

// ******************************** End of File ********************************
