/*  ________________________________________________________________________  */
/* |----------------------  P R O J E C T   N O T E S  ---------------------| */
/* |________________________________________________________________________| */
/* |                                                                        | */
/* |                           frydaysoft.de                                | */
/* |                                                                        | */
/* |________________________________________________________________________| */
/* | author     :   Sven Freitag                                            | */
/* |________________________________________________________________________| */
/* |****************************  S O U R C E  *****************************| */
/* |________________________________________________________________________| */
/* | project    :   LightCtrl                                               | */
/* |________________________________________________________________________| */
/* | file name  :   light.c          date: 12.11.2009      version :  1.0   | */
/* |________________________________________________________________________| */
/* | task       :   application                                             | */
/* |________________________________________________________________________| */
/* | target     :   ATMEL ATmega48/88, 3.6864 MHz                           | */
/* |________________________________________________________________________| */
/* | compiler   :   WIN AVR C Compiler                                      | */
/* |________________________________________________________________________| */
/* | history    :   01.09.2009  project start                               | */
/* |________________________________________________________________________| */
/* | tested by  :                                                           | */
/* |________________________________________________________________________| */


// *****************************************************************************
// ******************  Compiler and Preprocessor Directives  *******************
// *****************************************************************************


// *****************************************************************************
// ***************************  Include Directives  ****************************
// *****************************************************************************

#include "global.h"
#include "uart.h"
#include "SoftTimer.h"


// *****************************************************************************
// ******************  Module Global Definitions and Macros  *******************
// *****************************************************************************

#define SET_IRQ_TABLE   // fill IRQ-table with "reti" if defined

#define MK_LOWER    0x20
#define CMD_VER     'V' //'V' >1V 
#define CMD_LIGHT   'L' //'L' >3L?? 

#define POS_START 0
#define POS_LEN   1
#define POS_CMD   2
#define POS_DATA  3

#define RES_CMD_OK  '0'
#define RES_CMD_ERR '1'


// *****************************************************************************
// ********************  Module Global Variable Definitions  *******************
// *****************************************************************************

static UCHAR tx_buf[RX_MAX_LEN];
static UCHAR auc_version[] = { 'L','C',' ','1', '.', '0', '0' };
static UCHAR auc_conv_tbl[16] = {
    '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
};


// *****************************************************************************
// ********************  Module Local Function Prototypes  *********************
// *****************************************************************************

#define GetHexChar(idx) auc_conv_tbl[(idx)&0xF]

// PC0-PC3, PD4-PD7
#define InitLedPort()     PORTC &= 0xF0; DDRC |= 0x0F; PORTD &= 0x0F; DDRD |= 0xF0
#define SetLeds(val)      PORTC &= 0xF0; PORTC |= (val&0x0F); PORTD &= 0x0F; PORTD |= (val&0xF0)

// PB0 Out-CLK for latch
#define InitLatchClk()    PORTB &= 0xFE; DDRB |= 0x01
#define SetLatchClk(val)  PORTB &= 0xFE; PORTB |= (val&0x01)


#ifdef SET_IRQ_TABLE
//***** disable unused interrupts with "reti" !!!
//EMPTY_INTERRUPT(RESET_vect)         // 0: init9=reset
EMPTY_INTERRUPT(INT0_vect)            // 1:
EMPTY_INTERRUPT(INT1_vect)            // 2:
EMPTY_INTERRUPT(PCINT0_vect)          // 3:
EMPTY_INTERRUPT(PCINT1_vect)          // 4:
EMPTY_INTERRUPT(PCINT2_vect)          // 5:
EMPTY_INTERRUPT(WDT_vect)             // 6:
//EMPTY_INTERRUPT(TIMER2_COMPA_vect)  // 7: Timer2CompMatchAInt()
EMPTY_INTERRUPT(TIMER2_COMPB_vect)    // 8:
EMPTY_INTERRUPT(TIMER2_OVF_vect)      // 9:
EMPTY_INTERRUPT(TIMER1_CAPT_vect)     //10:
EMPTY_INTERRUPT(TIMER1_COMPA_vect)    //11:
EMPTY_INTERRUPT(TIMER1_COMPB_vect)    //12:
EMPTY_INTERRUPT(TIMER1_OVF_vect)      //13:
EMPTY_INTERRUPT(TIMER0_COMPA_vect)    //14:
EMPTY_INTERRUPT(TIMER0_COMPB_vect)    //15:
EMPTY_INTERRUPT(TIMER0_OVF_vect)      //16:
EMPTY_INTERRUPT(SPI_STC_vect)         //17:
//EMPTY_INTERRUPT(USART_RX_vect)      //18: Uart_Rx_Interrupt()
//EMPTY_INTERRUPT(USART_UDRE_vect)    //19: Uart_Tx_Empty_Interrupt()
EMPTY_INTERRUPT(USART_TX_vect)        //20:
EMPTY_INTERRUPT(ADC_vect)             //21:
EMPTY_INTERRUPT(EE_READY_vect)        //22:
EMPTY_INTERRUPT(ANALOG_COMP_vect)     //23:
EMPTY_INTERRUPT(TWI_vect)             //24:
EMPTY_INTERRUPT(SPM_READY_vect)       //25:
#endif


//******************************************************************************
//***                    ucDecodeHexNibble()                                 ***
//******************************************************************************
//***  utility function: convert ASCII-hex to BIN-hex (nibble)               ***
//******************************************************************************
//***  Function parameters:    ucDat (ASCII-hex value)                       ***
//******************************************************************************
//***  Function result:        binary nibble                                 ***
//******************************************************************************
//***  Author: Freitag                                                       ***
//******************************************************************************
static UCHAR ucDecodeHexNibble(UCHAR ucDat)
{
    UCHAR ucRet = ucDat;
    // a=61h, A=41h  '0'=30h
    if ( ucRet & 0x40 )
    {
        ucRet &= 0x07;
        ucRet += 9;
    }
    ucRet &= 0x0F;
    return ucRet;
}

//******************************************************************************
//***                      ucDecodeHexByte()                                 ***
//******************************************************************************
//***  utility function: convert ASCII-hex to BIN-hex                        ***
//******************************************************************************
//***  Function parameters:    pucBuf (pointer to 2 ASCII characters)        ***
//******************************************************************************
//***  Function result:        binary value                                  ***
//******************************************************************************
//***  Author: Freitag                                                       ***
//******************************************************************************
static UCHAR ucDecodeHexByte(UCHAR *pucBuf)
{
    UCHAR ucRet=0;

    if (pucBuf)
    { // a=61h, A=41h  '0'=30h
        ucRet = (ucDecodeHexNibble(*pucBuf) << 4) | ucDecodeHexNibble(*(pucBuf+1));
    } 
    return ucRet;
}



// *****************************************************************************
// *****************************  Global Functions  ****************************
// *****************************************************************************


//******************************************************************************
//***                                 main()                                 ***
//******************************************************************************
//***  Application task                                                      ***
//******************************************************************************
//***  Function parameters:    none                                          ***
//******************************************************************************
//***  Function result:        int (not used)                                ***
//******************************************************************************
//***  Author: Freitag                                                       ***
//******************************************************************************
int main (void)
{
    UCHAR uc_timer, uc_leds, x, chk_rx;
    BOOL b_rx_received = FALSE;

    // ***** setup system
    uc_leds = 0;
    InitLedPort();
    InitLatchClk();
    UartInit(BAUDRATE_57600);
    SetLeds(uc_leds);
    SetLatchClk(0);
    SetLatchClk(1);

#if 0
    // ***** simple test code
    uc_timer = 0;
    x = 0;
    while(1)
    {
      if ( !++x)
      if ( !++uc_timer)
      {
        SetLeds(uc_leds);
        SetLatchClk(0);
        SetLatchClk(1);
        uc_leds++;
      }
    }
#endif

    // ***** setup interrupts
    SoftTimerInit();
    enable_interrupt(); 

    // ***** print version after startup
    x = 1;
    if ( x )
    {
        for (x=0; x<sizeof(auc_version); x++)
        {
            tx_buf[x+POS_DATA] = auc_version[x];
        }
        tx_buf[POS_START] = START_TOK;
        tx_buf[POS_CMD] = CMD_VER;
        x += POS_DATA;
        tx_buf[POS_LEN] = GetHexChar(x-2);
        tx_buf[POS_CMD] += MK_LOWER;
        ComAddTailToken(tx_buf, x);
        bUartSendBlock(&tx_buf[0], x);
    }
    
    // ***** application loop
    uc_timer = ucSoftTimerGetTime1ms();
    chk_rx = 50;
    while(1)
    {
        // ***** 
        if (!chk_rx--)
        {
            chk_rx = 50;
            b_rx_received = bUartExistRxData();
        }
        else
        {
            if ( ucSoftTimerGetGetTimeDiff1ms(uc_timer) > 100 )
            { // ***** application task for 100ms (not used!!!)
                uc_timer = ucSoftTimerGetTime1ms();
                // ***** for future use...
#if 0
                // ***** simple test code for SW timer
                SetLeds(uc_leds);
                SetLatchClk(0);
                SetLatchClk(1);
                uc_leds++;
#endif
            }
        }

        // ***** handle incomming commands
        if ( b_rx_received )
        {   // ***** process new command if transmitter is still idle
            if ( bUartIsTxIdle() )
            {
                x = ucUartCopyRxData(&tx_buf[0]);
                b_rx_received = 0;
                switch(tx_buf[POS_CMD])
                {   // ***** 'V' >1V      return: version data
                    case CMD_VER:   
                        for (x=0; x<sizeof(auc_version); x++)
                        {
                            tx_buf[x+POS_DATA] = auc_version[x];
                        }
                        x += POS_DATA;
                        tx_buf[POS_LEN] = GetHexChar(x-2);
                        break;
                    // ***** //'L' >3L??   return: >3l:0
                    case CMD_LIGHT: 
                        uc_leds = ucDecodeHexByte(&tx_buf[POS_DATA]);
                        SetLeds(uc_leds);
                        SetLatchClk(0);
                        SetLatchClk(1);
                        tx_buf[POS_LEN] = '3';
                        tx_buf[POS_DATA] = ':';
                        tx_buf[POS_DATA+1] = RES_CMD_OK; // ok
                        x = 5;
                        break;
                    default:
                        x = 0;
                        break;
                }
                if ( x )
                {   // ***** send response
                    tx_buf[POS_CMD] += MK_LOWER;
                    ComAddTailToken(tx_buf, x);
                    bUartSendBlock(&tx_buf[0], x);
                }
            }
        }
    }

    return 0;
}

/********************************  End of File ********************************/
